@@ -37,13 +37,14 @@ HISTORY
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
+#include <fstream>
+#include <sstream>
+#include <iomanip>
+
 #include "FGFCS.h"
 #include "FGFDMExec.h"
 #include "FGGroundReactions.h"
 #include "input_output/FGPropertyManager.h"
-#include <fstream>
-#include <sstream>
-#include <iomanip>
 
 #include "models/flight_control/FGFilter.h"
 #include "models/flight_control/FGDeadBand.h"
@@ -63,7 +64,7 @@ using namespace std;
 
 namespace JSBSim {
 
-static const char *IdSrc = "$Id: FGFCS.cpp,v 1.74 2011/05/20 03:18:36 jberndt Exp $";
+static const char *IdSrc = "$Id: FGFCS.cpp,v 1.77 2011/09/25 14:05:40 bcoconni Exp $";
 static const char *IdHdr = ID_FCS;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -78,7 +79,7 @@ FGFCS::FGFCS(FGFDMExec* fdmex) : FGModel(fdmex)
   DaCmd = DeCmd = DrCmd = DsCmd = DfCmd = DsbCmd = DspCmd = 0;
   PTrimCmd = YTrimCmd = RTrimCmd = 0.0;
   GearCmd = GearPos = 1; // default to gear down
-  LeftBrake = RightBrake = CenterBrake = 0.0;
+  BrakePos.resize(FGLGear::bgNumBrakeGroups);
   TailhookPos = WingFoldPos = 0.0; 
 
   bind();
@@ -664,6 +665,8 @@ bool FGFCS::Load(Element* el, SystemType systype)
     channel_element = document->FindNextElement("channel");
   }
 
+  PostLoad(document, PropertyManager);
+
   ResetParser();
 
   return true;
@@ -673,17 +676,7 @@ bool FGFCS::Load(Element* el, SystemType systype)
 
 double FGFCS::GetBrake(FGLGear::BrakeGroup bg)
 {
-  switch (bg) {
-  case FGLGear::bgLeft:
-    return LeftBrake;
-  case FGLGear::bgRight:
-    return RightBrake;
-  case FGLGear::bgCenter:
-    return CenterBrake;
-  default:
-    cerr << "GetBrake asked to return a bogus brake value" << endl;
-  }
-  return 0.0;
+  return BrakePos[bg];
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -838,9 +831,10 @@ void FGFCS::AddThrottle(void)
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-void FGFCS::AddGear(void)
+void FGFCS::AddGear(unsigned int NumGear)
 {
-  SteerPosDeg.push_back(0.0);
+  SteerPosDeg.clear();
+  for (unsigned int i=0; i<NumGear; i++) SteerPosDeg.push_back(0.0);
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
