@@ -66,7 +66,7 @@ using namespace std;
 
 namespace JSBSim {
 
-static const char *IdSrc = "$Id: FGMSIS.cpp,v 1.17 2011/05/20 03:18:36 jberndt Exp $";
+static const char *IdSrc = "$Id: FGMSIS.cpp,v 1.18 2011/06/21 13:54:40 jberndt Exp $";
 static const char *IdHdr = ID_MSIS;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -140,16 +140,16 @@ bool MSIS::InitModel(void)
   input.f107 = 150.0;
   input.ap = 4.0;
 
-  UseInternal();
+//  UseInternal();
 
-  SLtemperature = intTemperature = 518.0;
-  SLpressure    = intPressure = 2116.7;
-  SLdensity     = intDensity = 0.002378;
-  SLsoundspeed  = sqrt(2403.0832 * SLtemperature);
-  rSLtemperature = 1.0/intTemperature;
-  rSLpressure    = 1.0/intPressure;
-  rSLdensity     = 1.0/intDensity;
-  rSLsoundspeed  = 1.0/SLsoundspeed;
+//  SLtemperature = intTemperature = 518.0;
+//  SLpressure    = intPressure = 2116.7;
+//  SLdensity     = intDensity = 0.002378;
+//  SLsoundspeed  = sqrt(2403.0832 * SLtemperature);
+//  rSLtemperature = 1.0/intTemperature;
+//  rSLpressure    = 1.0/intPressure;
+//  rSLdensity     = 1.0/intDensity;
+//  rSLsoundspeed  = 1.0/SLsoundspeed;
 
   return true;
 }
@@ -163,10 +163,10 @@ bool MSIS::Run(bool Holding)
 
   RunPreFunctions();
 
-  h = FDMExec->GetPropagate()->GetAltitudeASL();
+  double h = FDMExec->GetPropagate()->GetAltitudeASL();
 
   //do temp, pressure, and density first
-  if (!useExternal) {
+//  if (!useExternal) {
     // get sea-level values
     Calculate(FDMExec->GetAuxiliary()->GetDayOfYear(),
               FDMExec->GetAuxiliary()->GetSecondsInDay(),
@@ -188,14 +188,12 @@ bool MSIS::Run(bool Holding)
               h,
               FDMExec->GetPropagate()->GetLocation().GetLatitudeDeg(),
               FDMExec->GetPropagate()->GetLocation().GetLongitudeDeg());
-    intTemperature = output.t[1] * 1.8;
-    intDensity     = output.d[5] * 1.940321;
-    intPressure    = 1716.488 * intDensity * intTemperature;
+//    intTemperature = output.t[1] * 1.8;
+//    intDensity     = output.d[5] * 1.940321;
+//    intPressure    = 1716.488 * intDensity * intTemperature;
     //cout << "T=" << intTemperature << " D=" << intDensity << " P=";
     //cout << intPressure << " a=" << soundspeed << endl;
-  }
-
-  CalculateDerived();
+//  }
 
   RunPostFunctions();
 
@@ -1660,16 +1658,6 @@ void MSIS::Debug(int from)
   if (debug_lvl & 16) { // Sanity checking
   }
   if (debug_lvl & 32) { // Turbulence
-    if (first_pass && from == 2) {
-      cout << "vTurbulenceNED(X), vTurbulenceNED(Y), vTurbulenceNED(Z), "
-           << "vTurbulenceGrad(X), vTurbulenceGrad(Y), vTurbulenceGrad(Z), "
-           << "vDirection(X), vDirection(Y), vDirection(Z), "
-           << "Magnitude, "
-           << "vTurbPQR(P), vTurbPQR(Q), vTurbPQR(R), " << endl;
-    }
-    if (from == 2) {
-      cout << vTurbulenceNED << ", " << vTurbulenceGrad << ", " << vDirection << ", " << Magnitude << ", " << vTurbPQR << endl;
-    }
   }
   if (debug_lvl & 64) {
     if (from == 0) { // Constructor
