@@ -40,28 +40,25 @@ HISTORY
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
+#include <iostream>
 #include <sstream>
 
 #include "FGPiston.h"
-#include "models/FGAtmosphere.h"
-#include "models/FGAuxiliary.h"
-#include "models/FGPropulsion.h"
 #include "FGPropeller.h"
-#include <iostream>
 
 using namespace std;
 
 namespace JSBSim {
 
-static const char *IdSrc = "$Id: FGPiston.cpp,v 1.58 2011/06/13 15:23:09 jentron Exp $";
+static const char *IdSrc = "$Id: FGPiston.cpp,v 1.68 2011/10/11 15:13:34 jentron Exp $";
 static const char *IdHdr = ID_PISTON;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS IMPLEMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-FGPiston::FGPiston(FGFDMExec* exec, Element* el, int engine_number)
-  : FGEngine(exec, el, engine_number),
+FGPiston::FGPiston(FGFDMExec* exec, Element* el, int engine_number, struct Inputs& input)
+  : FGEngine(exec, el, engine_number, input),
   R_air(287.3),                  // Gas constant for air J/Kg/K
   rho_fuel(800),                 // estimate
   calorific_value_fuel(47.3e6),  // J/Kg
@@ -69,12 +66,13 @@ FGPiston::FGPiston(FGFDMExec* exec, Element* el, int engine_number)
   Cp_fuel(1700),
   standard_pressure(101320.73)
 {
+  Element *table_element;
   string token;
+  string name="";
 
   // Defaults and initializations
 
   Type = etPiston;
-  dt = FDMExec->GetDeltaT();
 
   // These items are read from the configuration file
   // Defaults are from a Lycoming O-360, more or less
@@ -104,6 +102,8 @@ FGPiston::FGPiston(FGFDMExec* exec, Element* el, int engine_number)
 
   // These are internal program variables
 
+  Lookup_Combustion_Efficiency = 0;
+  Mixture_Efficiency_Correlation = 0;
   crank_counter = 0;
   Magnetos = 0;
   minMAP = 21950;
@@ -135,39 +135,6 @@ FGPiston::FGPiston(FGFDMExec* exec, Element* el, int engine_number)
     BoostSwitchPressure[i] = 0.0;
   }
 
-  // First column is thi, second is neta (combustion efficiency)
-  Lookup_Combustion_Efficiency = new FGTable(12);
-  *Lookup_Combustion_Efficiency << 0.00 << 0.980;
-  *Lookup_Combustion_Efficiency << 0.90 << 0.980;
-  *Lookup_Combustion_Efficiency << 1.00 << 0.970;
-  *Lookup_Combustion_Efficiency << 1.05 << 0.950;
-  *Lookup_Combustion_Efficiency << 1.10 << 0.900;
-  *Lookup_Combustion_Efficiency << 1.15 << 0.850;
-  *Lookup_Combustion_Efficiency << 1.20 << 0.790;
-  *Lookup_Combustion_Efficiency << 1.30 << 0.700;
-  *Lookup_Combustion_Efficiency << 1.40 << 0.630;
-  *Lookup_Combustion_Efficiency << 1.50 << 0.570;
-  *Lookup_Combustion_Efficiency << 1.60 << 0.525;
-  *Lookup_Combustion_Efficiency << 2.00 << 0.345;
-
-  Mixture_Efficiency_Correlation = new FGTable(15);
-  *Mixture_Efficiency_Correlation << 0.05000 << 0.00000;
-  *Mixture_Efficiency_Correlation << 0.05137 << 0.00862;
-  *Mixture_Efficiency_Correlation << 0.05179 << 0.21552;
-  *Mixture_Efficiency_Correlation << 0.05430 << 0.48276;
-  *Mixture_Efficiency_Correlation << 0.05842 << 0.70690;
-  *Mixture_Efficiency_Correlation << 0.06312 << 0.83621;
-  *Mixture_Efficiency_Correlation << 0.06942 << 0.93103;
-  *Mixture_Efficiency_Correlation << 0.07786 << 1.00000;
-  *Mixture_Efficiency_Correlation << 0.08845 << 1.00000;
-  *Mixture_Efficiency_Correlation << 0.09270 << 0.98276;
-  *Mixture_Efficiency_Correlation << 0.10120 << 0.93103;
-  *Mixture_Efficiency_Correlation << 0.11455 << 0.72414;
-  *Mixture_Efficiency_Correlation << 0.12158 << 0.45690;
-  *Mixture_Efficiency_Correlation << 0.12435 << 0.23276;
-  *Mixture_Efficiency_Correlation << 0.12500 << 0.00000;
-
-
   // Read inputs from engine data file where present.
 
   if (el->FindElement("minmp")) // Should have ELSE statement telling default value used?
@@ -252,6 +219,21 @@ FGPiston::FGPiston(FGFDMExec* exec, Element* el, int engine_number)
       RatedAltitude[2] = el->FindElementValueAsNumberConvertTo("ratedaltitude3", "FT");
   }
 
+  while((table_element = el->FindNextElement("table")) != 0) {
+    name = table_element->GetAttributeValue("name");
+    try {
+      if (name == "COMBUSTION") {
+        Lookup_Combustion_Efficiency = new FGTable(PropertyManager, table_element);
+      } else if (name == "MIXTURE") {
+        Mixture_Efficiency_Correlation = new FGTable(PropertyManager, table_element);
+      } else {
+        cerr << "Unknown table type: " << name << " in piston engine definition." << endl;
+      }
+    } catch (std::string str) {
+      throw("Error loading piston engine table:" + name + ". " + str);
+    }
+  }
+
   StarterHP = sqrt(MaxHP) * 0.4;
   displacement_SI = Displacement * in3tom3;
   RatedMeanPistonSpeed_fps =  ( MaxRPM * Stroke) / (360); // AKA 2 * (RPM/60) * ( Stroke / 12) or 2NS
@@ -286,7 +268,6 @@ FGPiston::FGPiston(FGFDMExec* exec, Element* el, int engine_number)
  *
  *
  */
-
   if(Z_airbox < 0.0){
     double Ze=PeakMeanPistonSpeed_fps/RatedMeanPistonSpeed_fps; // engine impedence
     Z_airbox = (standard_pressure *Ze / maxMAP) - Ze; // impedence of airbox
@@ -295,6 +276,44 @@ FGPiston::FGPiston(FGFDMExec* exec, Element* el, int engine_number)
   Z_throttle=(PeakMeanPistonSpeed_fps/((IdleRPM * Stroke) / 360))*(standard_pressure/minMAP - 1) - Z_airbox; 
   //  Z_throttle=(MaxRPM/IdleRPM )*(standard_pressure/minMAP+2); // Constant for Throttle impedence
 
+// Default tables if not provided in the configuration file
+  if(Lookup_Combustion_Efficiency == 0) {
+    // First column is thi, second is neta (combustion efficiency)
+    Lookup_Combustion_Efficiency = new FGTable(12);
+    *Lookup_Combustion_Efficiency << 0.00 << 0.980;
+    *Lookup_Combustion_Efficiency << 0.90 << 0.980;
+    *Lookup_Combustion_Efficiency << 1.00 << 0.970;
+    *Lookup_Combustion_Efficiency << 1.05 << 0.950;
+    *Lookup_Combustion_Efficiency << 1.10 << 0.900;
+    *Lookup_Combustion_Efficiency << 1.15 << 0.850;
+    *Lookup_Combustion_Efficiency << 1.20 << 0.790;
+    *Lookup_Combustion_Efficiency << 1.30 << 0.700;
+    *Lookup_Combustion_Efficiency << 1.40 << 0.630;
+    *Lookup_Combustion_Efficiency << 1.50 << 0.570;
+    *Lookup_Combustion_Efficiency << 1.60 << 0.525;
+    *Lookup_Combustion_Efficiency << 2.00 << 0.345;
+  }
+
+    // First column is Fuel/Air Ratio, second is neta (mixture efficiency)
+  if( Mixture_Efficiency_Correlation == 0) {
+    Mixture_Efficiency_Correlation = new FGTable(15);
+    *Mixture_Efficiency_Correlation << 0.05000 << 0.00000;
+    *Mixture_Efficiency_Correlation << 0.05137 << 0.00862;
+    *Mixture_Efficiency_Correlation << 0.05179 << 0.21552;
+    *Mixture_Efficiency_Correlation << 0.05430 << 0.48276;
+    *Mixture_Efficiency_Correlation << 0.05842 << 0.70690;
+    *Mixture_Efficiency_Correlation << 0.06312 << 0.83621;
+    *Mixture_Efficiency_Correlation << 0.06942 << 0.93103;
+    *Mixture_Efficiency_Correlation << 0.07786 << 1.00000;
+    *Mixture_Efficiency_Correlation << 0.08845 << 1.00000;
+    *Mixture_Efficiency_Correlation << 0.09270 << 0.98276;
+    *Mixture_Efficiency_Correlation << 0.10120 << 0.93103;
+    *Mixture_Efficiency_Correlation << 0.11455 << 0.72414;
+    *Mixture_Efficiency_Correlation << 0.12158 << 0.45690;
+    *Mixture_Efficiency_Correlation << 0.12435 << 0.23276;
+    *Mixture_Efficiency_Correlation << 0.12500 << 0.00000;
+  }
+
   string property_name, base_property_name;
   base_property_name = CreateIndexedPropertyName("propulsion/engine", EngineNumber);
   property_name = base_property_name + "/power-hp";
@@ -317,8 +336,6 @@ FGPiston::FGPiston(FGFDMExec* exec, Element* el, int engine_number)
   PropertyManager->Tie(property_name, &BoostSpeed);
   property_name = base_property_name + "/cht-degF";
   PropertyManager->Tie(property_name, this, &FGPiston::getCylinderHeadTemp_degF);
-  property_name = base_property_name + "/engine-rpm";
-  PropertyManager->Tie(property_name, this, &FGPiston::getRPM);
   property_name = base_property_name + "/oil-temperature-degF";
   PropertyManager->Tie(property_name, this, &FGPiston::getOilTemp_degF);
   property_name = base_property_name + "/oil-pressure-psi";
@@ -347,13 +364,13 @@ FGPiston::FGPiston(FGFDMExec* exec, Element* el, int engine_number)
         // But we can also make a reasonable estimate, as below.
         BoostSwitchAltitude[i] = RatedAltitude[i] + 1000;
       }
-      BoostSwitchPressure[i] = Atmosphere->GetPressure(BoostSwitchAltitude[i]) * psftopa;
+      BoostSwitchPressure[i] = GetStdPressure100K(BoostSwitchAltitude[i]) * psftopa;
       //cout << "BoostSwitchAlt = " << BoostSwitchAltitude[i] << ", pressure = " << BoostSwitchPressure[i] << '\n';
       // Assume there is some hysteresis on the supercharger gear switch, and guess the value for now
       BoostSwitchHysteresis = 1000;
     }
     // Now work out the supercharger pressure multiplier of this speed from the rated boost and altitude.
-    RatedMAP[i] = Atmosphere->GetPressureSL() * psftopa + RatedBoost[i] * 6895;  // psi*6895 = Pa.
+    RatedMAP[i] = standard_pressure + RatedBoost[i] * 6895;  // psi*6895 = Pa.
     // Sometimes a separate BCV setting for takeoff or extra power is fitted.
     if (TakeoffBoost > RatedBoost[0]) {
       // Assume that the effect on the BCV is the same whichever speed is in use.
@@ -363,7 +380,7 @@ FGPiston::FGPiston(FGFDMExec* exec, Element* el, int engine_number)
       TakeoffMAP[i] = RatedMAP[i];
       bTakeoffBoost = false;
     }
-    BoostMul[i] = RatedMAP[i] / (Atmosphere->GetPressure(RatedAltitude[i]) * psftopa);
+    BoostMul[i] = RatedMAP[i] / (GetStdPressure100K(RatedAltitude[i]) * psftopa);
 
   }
 
@@ -391,10 +408,10 @@ void FGPiston::ResetToIC(void)
 {
   FGEngine::ResetToIC();
 
-  ManifoldPressure_inHg = Atmosphere->GetPressure() * psftoinhg; // psf to in Hg
-  MAP = Atmosphere->GetPressure() * psftopa;
+  ManifoldPressure_inHg = in.Pressure * psftoinhg; // psf to in Hg
+  MAP = in.Pressure * psftopa;
   TMAP = MAP;
-  double airTemperature_degK = RankineToKelvin(Atmosphere->GetTemperature());
+  double airTemperature_degK = RankineToKelvin(in.Temperature);
   OilTemp_degK = airTemperature_degK;
   CylinderHeadTemp_degK = airTemperature_degK;
   ExhaustGasTemp_degK = airTemperature_degK;
@@ -405,26 +422,26 @@ void FGPiston::ResetToIC(void)
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
 void FGPiston::Calculate(void)
 {
-  RunPreFunctions();
-
-  if (FuelFlow_gph > 0.0) ConsumeFuel();
-
-  Throttle = FCS->GetThrottlePos(EngineNumber);
-  Mixture = FCS->GetMixturePos(EngineNumber);
-
   // Input values.
 
-  p_amb = Atmosphere->GetPressure() * psftopa;
-  double p = Auxiliary->GetTotalPressure() * psftopa;
+  p_amb = in.Pressure * psftopa;
+  double p = in.TotalPressure * psftopa;
   p_ram = (p - p_amb) * Ram_Air_Factor + p_amb;
-  T_amb = RankineToKelvin(Atmosphere->GetTemperature());
+  T_amb = RankineToKelvin(in.Temperature);
+
+  RunPreFunctions();
+
+  TotalDeltaT = ( in.TotalDeltaT < 1e-9 ) ? 1.0 : in.TotalDeltaT;
+
+/* The thruster controls the engine RPM because it encapsulates the gear ratio and other transmission variables */
+  RPM = Thruster->GetEngineRPM();
 
-  RPM = Thruster->GetRPM() * Thruster->GetGearRatio();
   MeanPistonSpeed_fps =  ( RPM * Stroke) / (360); // AKA 2 * (RPM/60) * ( Stroke / 12) or 2NS
 
-  IAS = Auxiliary->GetVcalibratedKTS();
+  IAS = in.Vc;
 
   doEngineStartup();
   if (Boosted) doBoostControl();
@@ -449,10 +466,11 @@ void FGPiston::Calculate(void)
   doOilPressure();
 
   if (Thruster->GetType() == FGThruster::ttPropeller) {
-    ((FGPropeller*)Thruster)->SetAdvance(FCS->GetPropAdvance(EngineNumber));
-    ((FGPropeller*)Thruster)->SetFeather(FCS->GetPropFeather(EngineNumber));
+    ((FGPropeller*)Thruster)->SetAdvance(in.PropAdvance[EngineNumber]);
+    ((FGPropeller*)Thruster)->SetFeather(in.PropFeather[EngineNumber]);
   }
 
+  LoadThrusterInputs();
   Thruster->Calculate(HP * hptoftlbssec);
 
   RunPostFunctions();
@@ -462,22 +480,20 @@ void FGPiston::Calculate(void)
 
 double FGPiston::CalcFuelNeed(void)
 {
-  double dT = FDMExec->GetDeltaT() * Propulsion->GetRate();
-  FuelExpended = FuelFlowRate * dT;
+  FuelExpended = FuelFlowRate * in.TotalDeltaT;
+  if (!Starved) FuelUsedLbs += FuelExpended; 
   return FuelExpended;
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-int FGPiston::InitRunning(void) {
+int FGPiston::InitRunning(void)
+{
   Magnetos=3;
-  p_amb = Atmosphere->GetPressure() * psftopa;
-  double mix= p_amb / (101325.0*1.3);
-  FCS->SetMixturePos(EngineNumber, mix);
-  Thruster->SetRPM( 2.*IdleRPM/Thruster->GetGearRatio() );
-  //Thruster->SetRPM( 1000 );
-  Running=true;
-// cout <<"Set Running in FGPiston. RPM:" << Thruster->GetRPM()*Thruster->GetGearRatio() <<" Pressure:"<<p_amb<<" Mixture:"<< mix <<endl;
+  in.MixtureCmd[EngineNumber] = in.PressureRatio/1.3;
+  in.MixturePos[EngineNumber] = in.PressureRatio/1.3;
+  Thruster->SetRPM( 2.0*IdleRPM/Thruster->GetGearRatio() );
+  Running = true;
   return 1;
 }
 
@@ -597,13 +613,15 @@ void FGPiston::doBoostControl(void)
 
 void FGPiston::doMAP(void)
 {
-  double Zt = (1-Throttle)*(1-Throttle)*Z_throttle; // throttle impedence
+  double Zt = (1 - in.ThrottlePos[EngineNumber])*(1 - in.ThrottlePos[EngineNumber])*Z_throttle; // throttle impedence
   double Ze= MeanPistonSpeed_fps > 0 ? PeakMeanPistonSpeed_fps/MeanPistonSpeed_fps : 999999; // engine impedence
 
   double map_coefficient = Ze/(Ze+Z_airbox+Zt);
 
   // Add a one second lag to manifold pressure changes
-  double dMAP = (TMAP - p_ram * map_coefficient) * dt;
+  double dMAP=0;
+  dMAP = (TMAP - p_ram * map_coefficient) * TotalDeltaT;
+
   TMAP -=dMAP;
 
   // Find the mean effective pressure required to achieve this manifold pressure
@@ -620,7 +638,7 @@ void FGPiston::doMAP(void)
 
     bool bTakeoffPos = false;
     if (bTakeoffBoost) {
-      if (Throttle > 0.98) {
+      if (in.ThrottlePos[EngineNumber] > 0.98) {
         bTakeoffPos = true;
       }
     }
@@ -661,7 +679,7 @@ void FGPiston::doAirFlow(void)
 // loss of volumentric efficiency due to difference between MAP and exhaust pressure
 // Eq 6-10 from The Internal Combustion Engine - Charles Taylor Vol 1
   double ve =((gamma-1)/gamma) +( CompressionRatio -(p_amb/MAP))/(gamma*( CompressionRatio - 1));
-// FGAtmosphere::GetDensity() * FGJSBBase::m3toft3 / FGJSBBase::kgtoslug;
+
   rho_air = p_amb / (R_air * T_amb);
   double swept_volume = (displacement_SI * (RPM/60)) / 2;
   double v_dot_air = swept_volume * volumetric_efficiency *ve;
@@ -682,12 +700,18 @@ void FGPiston::doAirFlow(void)
 
 void FGPiston::doFuelFlow(void)
 {
-  double thi_sea_level = 1.3 * Mixture; // Allows an AFR of infinity:1 to 11.3075:1
+  double thi_sea_level = 1.3 * in.MixturePos[EngineNumber]; // Allows an AFR of infinity:1 to 11.3075:1
   equivalence_ratio = thi_sea_level * 101325.0 / p_amb;
-//  double AFR = 10+(12*(1-Mixture));// mixture 10:1 to 22:1
+//  double AFR = 10+(12*(1-in.Mixture[EngineNumber]));// mixture 10:1 to 22:1
 //  m_dot_fuel = m_dot_air / AFR;
   m_dot_fuel = (m_dot_air * equivalence_ratio) / 14.7;
   FuelFlowRate =  m_dot_fuel * 2.2046;  // kg to lb
+  if(Starved) // There is no fuel, so zero out the flows we've calculated so far
+  {
+    equivalence_ratio = 0.0;
+    FuelFlowRate = 0.0;
+    m_dot_fuel = 0.0;
+  }
   FuelFlow_pph = FuelFlowRate  * 3600;  // seconds to hours
   FuelFlow_gph = FuelFlow_pph / 6.0;    // Assumes 6 lbs / gallon
 }
@@ -771,11 +795,11 @@ void FGPiston::doEGT(void)
     heat_capacity_exhaust = (Cp_air * m_dot_air) + (Cp_fuel * m_dot_fuel);
     delta_T_exhaust = enthalpy_exhaust / heat_capacity_exhaust;
     ExhaustGasTemp_degK = T_amb + delta_T_exhaust;
-    ExhaustGasTemp_degK *= 0.444 + ((0.544 - 0.444) * PctPower);
   } else {  // Drop towards ambient - guess an appropriate time constant for now
     combustion_efficiency = 0;
-    dEGTdt = (RankineToKelvin(Atmosphere->GetTemperature()) - ExhaustGasTemp_degK) / 100.0;
-    delta_T_exhaust = dEGTdt * dt;
+    dEGTdt = (RankineToKelvin(in.Temperature) - ExhaustGasTemp_degK) / 100.0;
+    delta_T_exhaust = dEGTdt * TotalDeltaT;
+
     ExhaustGasTemp_degK += delta_T_exhaust;
   }
 }
@@ -814,7 +838,8 @@ void FGPiston::doCHT(void)
   double HeatCapacityCylinderHead = CpCylinderHead * MassCylinderHead;
 
   CylinderHeadTemp_degK +=
-    (dqdt_cylinder_head / HeatCapacityCylinderHead) * dt;
+    (dqdt_cylinder_head / HeatCapacityCylinderHead) * TotalDeltaT;
+
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -848,7 +873,7 @@ void FGPiston::doOilTemperature(void)
 
   double dOilTempdt = (target_oil_temp - OilTemp_degK) / time_constant;
 
-  OilTemp_degK += (dOilTempdt * dt);
+  OilTemp_degK += (dOilTempdt * TotalDeltaT);
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -876,6 +901,30 @@ void FGPiston::doOilPressure(void)
   OilPressure_psi += (Design_Oil_Temp - OilTemp_degK) * Oil_Viscosity_Index * OilPressure_psi / Oil_Press_Relief_Valve;
 }
 
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+//
+// This is a local copy of the same function in FGStandardAtmosphere.
+
+double FGPiston::GetStdPressure100K(double altitude) const
+{
+  // Limit this equation to input altitudes of 100000 ft.
+  if (altitude > 100000.0) altitude = 100000.0;
+
+  double alt[5];
+  const double coef[5] = {  2116.217,
+                          -7.648932746E-2,
+                           1.0925498604E-6,
+                          -7.1135726027E-12,
+                           1.7470331356E-17 };
+
+  alt[0] = 1;
+  for (int pwr=1; pwr<=4; pwr++) alt[pwr] = alt[pwr-1]*altitude;
+
+  double press = 0.0;
+  for (int ctr=0; ctr<=4; ctr++) press += coef[ctr]*alt[ctr];
+  return press;
+}
+
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 string FGPiston::GetEngineLabels(const string& delimiter)
