@@ -56,9 +56,12 @@
 #include <simgear/debug/logstream.hxx>
 #include <simgear/structure/exception.hxx>
 #include <simgear/structure/event_mgr.hxx>
+#include <simgear/structure/SGPerfMon.hxx>
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/misc/sg_dir.hxx>
 #include <simgear/misc/sgstream.hxx>
+#include <simgear/misc/strutils.hxx>
+#include <simgear/props/props_io.hxx>
 
 #include <simgear/misc/interpolator.hxx>
 #include <simgear/scene/material/matlib.hxx>
@@ -75,6 +78,7 @@
 #include <AIModel/AIManager.hxx>
 
 #include <ATCDCL/ATCmgr.hxx>
+#include <ATC/atc_mgr.hxx>
 
 #include <Autopilot/route_mgr.hxx>
 #include <Autopilot/autopilotgroup.hxx>
@@ -103,7 +107,7 @@
 #include <Traffic/TrafficMgr.hxx>
 #include <MultiPlayer/multiplaymgr.hxx>
 #include <FDM/fdm_shell.hxx>
-
+#include <Environment/ephemeris.hxx>
 #include <Environment/environment_mgr.hxx>
 
 #include "fg_init.hxx"
@@ -118,225 +122,9 @@
 #include "main.hxx"
 
 
-#ifdef __APPLE__
-#  include <CoreFoundation/CoreFoundation.h>
-#endif
-
 using std::string;
 using namespace boost::algorithm;
 
-extern const char *default_root;
-
-
-// Scan the command line options for the specified option and return
-// the value.
-static string fgScanForOption( const string& option, int argc, char **argv ) {
-    int i = 1;
-
-    if (hostname == NULL)
-    {
-        char _hostname[256];
-        if( gethostname(_hostname, 256) >= 0 ) {
-            hostname = strdup(_hostname);
-            free_hostname = true;
-        }
-    }
-
-    SG_LOG(SG_GENERAL, SG_INFO, "Scanning command line for: " << option );
-
-    int len = option.length();
-
-    while ( i < argc ) {
-	SG_LOG( SG_GENERAL, SG_DEBUG, "argv[" << i << "] = " << argv[i] );
-
-	string arg = argv[i];
-	if ( arg.find( option ) == 0 ) {
-	    return arg.substr( len );
-	}
-
-	i++;
-    }
-
-    return "";
-}
-
-
-// Scan the user config files for the specified option and return
-// the value.
-static string fgScanForOption( const string& option, const string& path ) {
-    sg_gzifstream in( path );
-    if ( !in.is_open() ) {
-        return "";
-    }
-
-    SG_LOG( SG_GENERAL, SG_INFO, "Scanning " << path << " for: " << option );
-
-    int len = option.length();
-
-    in >> skipcomment;
-    while ( ! in.eof() ) {
-	string line;
-	getline( in, line, '\n' );
-
-        // catch extraneous (DOS) line ending character
-        if ( line[line.length() - 1] < 32 ) {
-            line = line.substr( 0, line.length()-1 );
-        }
-
-	if ( line.find( option ) == 0 ) {
-	    return line.substr( len );
-	}
-
-	in >> skipcomment;
-    }
-
-    return "";
-}
-
-// Scan the user config files for the specified option and return
-// the value.
-static string fgScanForOption( const string& option ) {
-    string arg("");
-
-#if defined( unix ) || defined( __CYGWIN__ ) || defined(_MSC_VER)
-    // Next check home directory for .fgfsrc.hostname file
-    if ( arg.empty() ) {
-        if ( homedir != NULL && hostname != NULL && strlen(hostname) > 0) {
-            SGPath config( homedir );
-            config.append( ".fgfsrc" );
-            config.concat( "." );
-            config.concat( hostname );
-            arg = fgScanForOption( option, config.str() );
-        }
-    }
-#endif
-
-    // Next check home directory for .fgfsrc file
-    if ( arg.empty() ) {
-        if ( homedir != NULL ) {
-            SGPath config( homedir );
-            config.append( ".fgfsrc" );
-            arg = fgScanForOption( option, config.str() );
-        }
-    }
-
-    if ( arg.empty() ) {
-        // Check for $fg_root/system.fgfsrc
-        SGPath config( globals->get_fg_root() );
-        config.append( "system.fgfsrc" );
-        arg = fgScanForOption( option, config.str() );
-    }
-
-    return arg;
-}
-
-
-// Read in configuration (files and command line options) but only set
-// fg_root and aircraft_paths, which are needed *before* do_options() is called
-// in fgInitConfig
-
-bool fgInitFGRoot ( int argc, char **argv ) {
-    string root;
-
-    // First parse command line options looking for --fg-root=, this
-    // will override anything specified in a config file
-    root = fgScanForOption( "--fg-root=", argc, argv);
-
-    // Check in one of the user configuration files.
-    if (root.empty() )
-        root = fgScanForOption( "--fg-root=" );
-    
-    // Next check if fg-root is set as an env variable
-    if ( root.empty() ) {
-        char *envp = ::getenv( "FG_ROOT" );
-        if ( envp != NULL ) {
-            root = envp;
-        }
-    }
-
-    // Otherwise, default to a random compiled-in location if we can't
-    // find fg-root any other way.
-    if ( root.empty() ) {
-#if defined( __CYGWIN__ )
-        root = "../data";
-#elif defined( _WIN32 )
-        root = "..\\data";
-#elif defined(__APPLE__) 
-        /*
-        The following code looks for the base package inside the application 
-        bundle, in the standard Contents/Resources location. 
-        */
-        CFURLRef resourcesUrl = CFBundleCopyResourcesDirectoryURL(CFBundleGetMainBundle());
-
-        // look for a 'data' subdir
-        CFURLRef dataDir = CFURLCreateCopyAppendingPathComponent(NULL, resourcesUrl, CFSTR("data"), true);
-
-        // now convert down to a path, and the a c-string
-        CFStringRef path = CFURLCopyFileSystemPath(dataDir, kCFURLPOSIXPathStyle);
-        root = CFStringGetCStringPtr(path, CFStringGetSystemEncoding());
-
-        CFRelease(resourcesUrl);
-        CFRelease(dataDir);
-        CFRelease(path);
-#else
-        root = PKGLIBDIR;
-#endif
-    }
-
-    SG_LOG(SG_INPUT, SG_INFO, "fg_root = " << root );
-    globals->set_fg_root(root);
-    
-    return true;
-}
-
-
-// Read in configuration (files and command line options) but only set
-// aircraft
-bool fgInitFGAircraft ( int argc, char **argv ) {
-    
-    string aircraftDir = fgScanForOption("--fg-aircraft=", argc, argv);
-    if (aircraftDir.empty()) {
-      aircraftDir =  fgScanForOption("--fg-aircraft="); 
-    }
-
-    const char* envp = ::getenv("FG_AIRCRAFT");
-    if (aircraftDir.empty() && envp) {
-      globals->append_aircraft_paths(envp);
-    }
-    
-    if (!aircraftDir.empty()) {
-      globals->append_aircraft_paths(aircraftDir);
-    }
-    
-    string aircraft;
-
-    // First parse command line options looking for --aircraft=, this
-    // will override anything specified in a config file
-    aircraft = fgScanForOption( "--aircraft=", argc, argv );
-    if ( aircraft.empty() ) {
-        // check synonym option
-        aircraft = fgScanForOption( "--vehicle=", argc, argv );
-    }
-
-    // Check in one of the user configuration files.
-    if ( aircraft.empty() ) {
-        aircraft = fgScanForOption( "--aircraft=" );
-    }
-    if ( aircraft.empty() ) {
-        aircraft = fgScanForOption( "--vehicle=" );
-    }
-
-    // if an aircraft was specified, set the property name
-    if ( !aircraft.empty() ) {
-        SG_LOG(SG_INPUT, SG_INFO, "aircraft = " << aircraft );
-        fgSetString("/sim/aircraft", aircraft.c_str() );
-    } else {
-        SG_LOG(SG_INPUT, SG_INFO, "No user specified aircraft, using default" );
-    }
-
-    return true;
-}
-
 
 // Return the current base package version
 string fgBasePackageVersion() {
@@ -474,45 +262,6 @@ bool fgDetectLanguage() {
     return true;
 }
 
-// Attempt to locate and parse the various non-XML config files in order
-// from least precidence to greatest precidence
-static void
-do_options (int argc, char ** argv)
-{
-    // Check for $fg_root/system.fgfsrc
-    SGPath config( globals->get_fg_root() );
-    config.append( "system.fgfsrc" );
-    fgParseOptions(config.str());
-
-#if defined( unix ) || defined( __CYGWIN__ ) || defined(_MSC_VER)
-    if( hostname != NULL && strlen(hostname) > 0 ) {
-        config.concat( "." );
-        config.concat( hostname );
-        fgParseOptions(config.str());
-    }
-#endif
-
-    // Check for ~/.fgfsrc
-    if ( homedir != NULL ) {
-        config.set( homedir );
-        config.append( ".fgfsrc" );
-        fgParseOptions(config.str());
-    }
-
-#if defined( unix ) || defined( __CYGWIN__ ) || defined(_MSC_VER)
-    if( hostname != NULL && strlen(hostname) > 0 ) {
-        // Check for ~/.fgfsrc.hostname
-        config.concat( "." );
-        config.concat( hostname );
-        fgParseOptions(config.str());
-    }
-#endif
-
-    // Parse remaining command line options
-    // These will override anything specified in a config file
-    fgParseArgs(argc, argv);
-}
-
 template <class T>
 bool fgFindAircraftInDir(const SGPath& dirPath, T* obj, bool (T::*pred)(const SGPath& p))
 {
@@ -590,6 +339,29 @@ public:
     }
     
     _searchAircraft = aircraft + "-set.xml";
+    std::string aircraftDir = fgGetString("/sim/aircraft-dir", "");
+    if (!aircraftDir.empty()) {
+      // aircraft-dir was set, skip any searching at all, if it's valid
+      simgear::Dir acPath(aircraftDir);
+      SGPath setFile = acPath.file(_searchAircraft);
+      if (setFile.exists()) {
+        SG_LOG(SG_GENERAL, SG_INFO, "found aircraft in dir: " << aircraftDir );
+        
+        try {
+          readProperties(setFile.str(), globals->get_props());
+        } catch ( const sg_exception &e ) {
+          SG_LOG(SG_INPUT, SG_ALERT, "Error reading aircraft: " << e.getFormattedMessage());
+          return false;
+        }
+        
+        return true;
+      } else {
+        SG_LOG(SG_GENERAL, SG_ALERT, "aircraft '" << _searchAircraft << 
+               "' not found in specified dir:" << aircraftDir);
+        return false;
+      }
+    }
+    
     if (!checkCache()) {
       // prepare cache for re-scan
       SGPropertyNode *n = _cache->getNode("fg-root", true);
@@ -703,12 +475,69 @@ private:
   SGPropertyNode* _cache;
 };
 
+#ifdef _WIN32
+static SGPath platformDefaultDataPath()
+{
+  char *envp = ::getenv( "APPDATA" );
+  SGPath config( envp );
+  config.append( "flightgear.org" );
+  return config;
+}
+#elif __APPLE__
+
+#include <CoreServices/CoreServices.h>
+
+static SGPath platformDefaultDataPath()
+{
+  FSRef ref;
+  OSErr err = FSFindFolder(kUserDomain, kApplicationSupportFolderType, false, &ref);
+  if (err) {
+    return SGPath();
+  }
+  
+  unsigned char path[1024];
+  if (FSRefMakePath(&ref, path, 1024) != noErr) {
+    return SGPath();
+  }
+  
+  SGPath appData;
+  appData.set((const char*) path);
+  appData.append("FlightGear");
+  return appData;
+}
+#else
+static SGPath platformDefaultDataPath()
+{
+  SGPath config( homedir );
+  config.append( ".fgfs" );
+  return config;
+}
+#endif
+
 // Read in configuration (file and command line)
-bool fgInitConfig ( int argc, char **argv ) {
-
-    // First, set some sane default values
-    fgSetDefaults();
-
+bool fgInitConfig ( int argc, char **argv )
+{
+    SGPath dataPath = platformDefaultDataPath();
+    
+    const char *fg_home = getenv("FG_HOME");
+    if (fg_home)
+      dataPath = fg_home;
+    
+    simgear::Dir exportDir(simgear::Dir(dataPath).file("Export"));
+    if (!exportDir.exists()) {
+      exportDir.create(0777);
+    }
+    
+    // Set /sim/fg-home and don't allow malign code to override it until
+    // Nasal security is set up.  Use FG_HOME if necessary.
+    SGPropertyNode *home = fgGetNode("/sim", true);
+    home->removeChild("fg-home", 0, false);
+    home = home->getChild("fg-home", 0, true);
+    home->setStringValue(dataPath.c_str());
+    home->setAttribute(SGPropertyNode::WRITE, false);
+  
+    flightgear::Options::sharedInstance()->init(argc, argv, dataPath);
+  
     // Read global preferences from $FG_ROOT/preferences.xml
     SG_LOG(SG_INPUT, SG_INFO, "Reading global preferences");
     fgLoadProps("preferences.xml", globals->get_props());
@@ -720,46 +549,22 @@ bool fgInitConfig ( int argc, char **argv ) {
     }
 
     SGPropertyNode autosave;
-#ifdef _WIN32
-    char *envp = ::getenv( "APPDATA" );
-    if (envp != NULL ) {
-        SGPath config( envp );
-        config.append( "flightgear.org" );
-#else
-    if ( homedir != NULL ) {
-        SGPath config( homedir );
-        config.append( ".fgfs" );
-#endif
-        const char *fg_home = getenv("FG_HOME");
-        if (fg_home)
-            config = fg_home;
 
-        SGPath home_export(config.str());
-        home_export.append("Export/dummy");
-        home_export.create_dir(0777);
 
-        // Set /sim/fg-home and don't allow malign code to override it until
-        // Nasal security is set up.  Use FG_HOME if necessary.
-        SGPropertyNode *home = fgGetNode("/sim", true);
-        home->removeChild("fg-home", 0, false);
-        home = home->getChild("fg-home", 0, true);
-        home->setStringValue(config.c_str());
-        home->setAttribute(SGPropertyNode::WRITE, false);
-
-        config.append( "autosave.xml" );
-        if (config.exists()) {
-          SG_LOG(SG_INPUT, SG_INFO, "Reading user settings from " << config.str());
-          try {
-              readProperties(config.str(), &autosave, SGPropertyNode::USERARCHIVE);
-          } catch (sg_exception& e) {
-              SG_LOG(SG_INPUT, SG_WARN, "failed to read user settings:" << e.getMessage()
-                << "(from " << e.getOrigin() << ")");
-          }
-        }
+    SGPath autosaveFile = simgear::Dir(dataPath).file("autosave.xml");
+    if (autosaveFile.exists()) {
+      SG_LOG(SG_INPUT, SG_INFO, "Reading user settings from " << autosaveFile.str());
+      try {
+          readProperties(autosaveFile.str(), &autosave, SGPropertyNode::USERARCHIVE);
+      } catch (sg_exception& e) {
+          SG_LOG(SG_INPUT, SG_WARN, "failed to read user settings:" << e.getMessage()
+            << "(from " << e.getOrigin() << ")");
+      }
     }
-    
+
     // Scan user config files and command line for a specified aircraft.
-    fgInitFGAircraft(argc, argv);
+    flightgear::Options::sharedInstance()->initAircraft();
+
     FindAndCacheAircraft f(&autosave);
     if (!f.loadAircraft()) {
       return false;
@@ -767,10 +572,14 @@ bool fgInitConfig ( int argc, char **argv ) {
 
     copyProperties(&autosave, globals->get_props());
 
+    // TODO Move some of the code above into loadUserSettings after the 2.6 release
+    // call dummy function, for now just to indicate that data was loaded
+    globals->loadUserSettings();
+
     // parse options after loading aircraft to ensure any user
     // overrides of defaults are honored.
-    do_options(argc, argv);
-
+    flightgear::Options::sharedInstance()->processOptions();
+      
     return true;
 }
 
@@ -898,18 +707,65 @@ static bool fgSetPosFromAirportIDandParkpos( const string& id, const string& par
     FGAirportDynamics* dcs = apt->getDynamics();
     if (!dcs) {
         SG_LOG( SG_GENERAL, SG_ALERT,
-                "Failed to find parking position " << parkpos <<
-                " at airport " << id );
+                "Airport " << id << "does not appear to have parking information available");
         return false;
     }
     
     int park_index = dcs->getNrOfParkings() - 1;
-    while (park_index >= 0 && dcs->getParkingName(park_index) != parkpos) park_index--;
-    if (park_index < 0) {
-        SG_LOG( SG_GENERAL, SG_ALERT,
-                "Failed to find parking position " << parkpos <<
-                " at airport " << id );
-        return false;
+    bool succes;
+    double radius = fgGetDouble("/sim/dimensions/radius-m");
+    if ((parkpos == string("AVAILABLE")) && (radius > 0)) {
+        double lat, lon, heading;
+        string fltType;
+        string acOperator;
+        SGPath acData;
+        try {
+            acData = fgGetString("/sim/fg-home");
+            acData.append("aircraft-data");
+            string acfile = fgGetString("/sim/aircraft") + string(".xml");
+            acData.append(acfile);
+            SGPropertyNode root;
+            readProperties(acData.str(), &root);
+            SGPropertyNode * node = root.getNode("sim");
+            fltType    = node->getStringValue("aircraft-class", "NONE"     );
+            acOperator = node->getStringValue("aircraft-operator", "NONE"     );
+        } catch (const sg_exception &) {
+            SG_LOG(SG_GENERAL, SG_INFO,
+                "Could not load aircraft aircrat type and operator information from: " << acData.str() << ". Using defaults");
+
+       // cout << path.str() << endl;
+        }
+        if (fltType.empty() || fltType == "NONE") {
+            SG_LOG(SG_GENERAL, SG_INFO,
+                "Aircraft type information not found in: " << acData.str() << ". Using default value");
+                fltType = fgGetString("/sim/aircraft-class"   );
+        }
+        if (acOperator.empty() || fltType == "NONE") {
+            SG_LOG(SG_GENERAL, SG_INFO,
+                "Aircraft operator information not found in: " << acData.str() << ". Using default value");
+                acOperator = fgGetString("/sim/aircraft-operator"   );
+        }
+
+        cerr << "Running aircraft " << fltType << " of livery " << acOperator << endl;
+        string acType; // Currently not used by findAvailable parking, so safe to leave empty. 
+        succes = dcs->getAvailableParking(&lat, &lon, &heading, &park_index, radius, fltType, acType, acOperator);
+        if (succes) {
+            fgGetString("/sim/presets/parkpos");
+            fgSetString("/sim/presets/parkpos", dcs->getParking(park_index)->getName());
+        } else {
+            SG_LOG( SG_GENERAL, SG_ALERT,
+                    "Failed to find a suitable parking at airport " << id );
+            return false;
+        }
+    } else {
+        //cerr << "We shouldn't get here when AVAILABLE" << endl;
+        while (park_index >= 0 && dcs->getParkingName(park_index) != parkpos) park_index--;
+        if (park_index < 0) {
+            SG_LOG( SG_GENERAL, SG_ALERT,
+                    "Failed to find parking position " << parkpos <<
+                    " at airport " << id );
+            return false;
+        }
     }
     FGParking* parking = dcs->getParking(park_index);
     parking->setAvailable(false);
@@ -996,18 +852,37 @@ static void fgSetDistOrAltFromGlideSlope() {
 
 
 // Set current_options lon/lat given an airport id and heading (degrees)
-static bool fgSetPosFromNAV( const string& id, const double& freq ) {
-    FGNavRecord *nav
-        = globals->get_navlist()->findByIdentAndFreq( id.c_str(), freq );
+static bool fgSetPosFromNAV( const string& id, const double& freq, FGPositioned::Type type ) {
 
-  if (!nav) {
-    SG_LOG( SG_GENERAL, SG_ALERT, "Failed to locate NAV = "
-                << id << ":" << freq );
-    return false;
-  }
-  
-  fgApplyStartOffset(nav->geod(), fgGetDouble("/sim/presets/heading-deg"));
-  return true;
+    const nav_list_type navlist
+        = globals->get_navlist()->findByIdentAndFreq( id.c_str(), freq, type );
+
+    if (navlist.size() == 0 ) {
+        SG_LOG( SG_GENERAL, SG_ALERT, "Failed to locate NAV = "
+            << id << ":" << freq );
+        return false;
+    }
+
+    if( navlist.size() > 1 ) {
+        ostringstream buf;
+        buf << "Ambigous NAV-ID: '" << id << "'. Specify id and frequency. Available stations:" << endl;
+        for( nav_list_type::const_iterator it = navlist.begin(); it != navlist.end(); ++it ) {
+            // NDB stored in kHz, VOR stored in MHz * 100 :-P
+            double factor = (*it)->type() == FGPositioned::NDB ? 1.0 : 1/100.0;
+            string unit = (*it)->type() == FGPositioned::NDB ? "kHz" : "MHz";
+            buf << (*it)->ident() << " "
+                << setprecision(5) << (double)((*it)->get_freq() * factor) << " "
+                << (*it)->get_lat() << "/" << (*it)->get_lon()
+                << endl;
+        }
+
+        SG_LOG( SG_GENERAL, SG_ALERT, buf.str() );
+        return false;
+    }
+
+    FGNavRecord *nav = navlist[0];
+    fgApplyStartOffset(nav->geod(), fgGetDouble("/sim/presets/heading-deg"));
+    return true;
 }
 
 // Set current_options lon/lat given an aircraft carrier id
@@ -1219,14 +1094,14 @@ bool fgInitPosition() {
 
     if ( !set_pos && !vor.empty() ) {
         // a VOR is requested
-        if ( fgSetPosFromNAV( vor, vor_freq ) ) {
+        if ( fgSetPosFromNAV( vor, vor_freq, FGPositioned::VOR ) ) {
             set_pos = true;
         }
     }
 
     if ( !set_pos && !ndb.empty() ) {
         // an NDB is requested
-        if ( fgSetPosFromNAV( ndb, ndb_freq ) ) {
+        if ( fgSetPosFromNAV( ndb, ndb_freq, FGPositioned::NDB ) ) {
             set_pos = true;
         }
     }
@@ -1306,16 +1181,19 @@ bool fgInitGeneral() {
 // gear, its initialization call should located in this routine.
 // Returns non-zero if a problem encountered.
 bool fgInitSubsystems() {
-    // static const SGPropertyNode *longitude
-    //     = fgGetNode("/sim/presets/longitude-deg");
-    // static const SGPropertyNode *latitude
-    //     = fgGetNode("/sim/presets/latitude-deg");
-    // static const SGPropertyNode *altitude
-    //     = fgGetNode("/sim/presets/altitude-ft");
 
     SG_LOG( SG_GENERAL, SG_INFO, "Initialize Subsystems");
     SG_LOG( SG_GENERAL, SG_INFO, "========== ==========");
 
+    ////////////////////////////////////////////////////////////////////
+    // Initialize the sound subsystem.
+    ////////////////////////////////////////////////////////////////////
+    // Sound manager uses an own subsystem group "SOUND" which is the last
+    // to be updated in every loop.
+    // Sound manager is updated last so it can use the CPU while the GPU
+    // is processing the scenery (doubled the frame-rate for me) -EMH-
+    globals->add_subsystem("sound", new SGSoundMgr, SGSubsystemMgr::SOUND);
+
     ////////////////////////////////////////////////////////////////////
     // Initialize the event manager subsystem.
     ////////////////////////////////////////////////////////////////////
@@ -1335,15 +1213,24 @@ bool fgInitSubsystems() {
     ////////////////////////////////////////////////////////////////////
     globals->add_subsystem("properties", new FGProperties);
 
+
+    ////////////////////////////////////////////////////////////////////
+    // Add the performance monitoring system.
+    ////////////////////////////////////////////////////////////////////
+    globals->add_subsystem("performance-mon",
+            new SGPerformanceMonitor(globals->get_subsystem_mgr(),
+                                     fgGetNode("/sim/performance-monitor", true)));
+
     ////////////////////////////////////////////////////////////////////
     // Initialize the material property subsystem.
     ////////////////////////////////////////////////////////////////////
 
     SGPath mpath( globals->get_fg_root() );
-    mpath.append( "materials.xml" );
+    mpath.append( fgGetString("/sim/rendering/materials-file") );
     if ( ! globals->get_matlib()->load(globals->get_fg_root(), mpath.str(),
             globals->get_props()) ) {
-        SG_LOG( SG_GENERAL, SG_ALERT, "Error loading material lib!" );
+        SG_LOG( SG_GENERAL, SG_ALERT,
+                "Error loading materials file " << mpath.str() );
         exit(-1);
     }
 
@@ -1368,7 +1255,8 @@ bool fgInitSubsystems() {
 
     // Initialize the weather modeling subsystem
     globals->add_subsystem("environment", new FGEnvironmentMgr);
-
+    globals->add_subsystem("ephemeris", new Ephemeris);
+    
     ////////////////////////////////////////////////////////////////////
     // Initialize the aircraft systems and instrumentation (before the
     // autopilot.)
@@ -1410,11 +1298,23 @@ bool fgInitSubsystems() {
 
     ////////////////////////////////////////////////////////////////////
     // Initialise the ATC Manager
+    // Note that this is old stuff, but might be necessesary for the 
+    // current ATIS implementation. Therefore, leave it in here
+    // until the ATIS system is ported over to make use of the ATIS 
+    // sub system infrastructure.
     ////////////////////////////////////////////////////////////////////
 
-    SG_LOG(SG_GENERAL, SG_INFO, "  ATC Manager");
-    globals->set_ATC_mgr(new FGATCMgr);
-    globals->get_ATC_mgr()->init(); 
+    globals->add_subsystem("ATC-old", new FGATCMgr, SGSubsystemMgr::INIT);
+
+    ////////////////////////////////////////////////////////////////////
+   // Initialize the ATC subsystem
+    ////////////////////////////////////////////////////////////////////
+    globals->add_subsystem("ATC", new FGATCManager, SGSubsystemMgr::POST_FDM);
+    ////////////////////////////////////////////////////////////////////
+    // Initialise the ATIS Subsystem
+    ////////////////////////////////////////////////////////////////////
+    //globals->add_subsystem("atis", new FGAtisManager, SGSubsystemMgr::POST_FDM);
+
 
     ////////////////////////////////////////////////////////////////////
     // Initialize multiplayer subsystem
@@ -1426,14 +1326,14 @@ bool fgInitSubsystems() {
     // Initialise the AI Model Manager
     ////////////////////////////////////////////////////////////////////
     SG_LOG(SG_GENERAL, SG_INFO, "  AI Model Manager");
-    globals->add_subsystem("ai_model", new FGAIManager, SGSubsystemMgr::POST_FDM);
-    globals->add_subsystem("submodel_mgr", new FGSubmodelMgr, SGSubsystemMgr::POST_FDM);
+    globals->add_subsystem("ai-model", new FGAIManager, SGSubsystemMgr::POST_FDM);
+    globals->add_subsystem("submodel-mgr", new FGSubmodelMgr, SGSubsystemMgr::POST_FDM);
 
 
     // It's probably a good idea to initialize the top level traffic manager
     // After the AI and ATC systems have been initialized properly.
     // AI Traffic manager
-    globals->add_subsystem("Traffic Manager", new FGTrafficManager, SGSubsystemMgr::POST_FDM);
+    globals->add_subsystem("traffic-manager", new FGTrafficManager, SGSubsystemMgr::POST_FDM);
 
     ////////////////////////////////////////////////////////////////////
     // Add a new 2D panel.
@@ -1568,7 +1468,7 @@ void fgReInitSubsystems()
     
     // Force reupdating the positions of the ai 3d models. They are used for
     // initializing ground level for the FDM.
-    globals->get_subsystem("ai_model")->reinit();
+    globals->get_subsystem("ai-model")->reinit();
 
     // Initialize the FDM
     globals->get_subsystem("flight")->reinit();
