@@ -47,11 +47,11 @@
 #include <simgear/props/AtomicChangeListener.hxx>
 #include <simgear/props/props.hxx>
 #include <simgear/timing/sg_time.hxx>
+#include <simgear/timing/timestamp.hxx>
 #include <simgear/magvar/magvar.hxx>
 #include <simgear/math/sg_random.h>
 #include <simgear/io/raw_socket.hxx>
 #include <simgear/scene/tsync/terrasync.hxx>
-#include <simgear/misc/sg_sleep.hxx>
 
 #include <Time/light.hxx>
 #include <Aircraft/replay.hxx>
@@ -67,7 +67,6 @@
 #include <ATCDCL/ATCmgr.hxx>
 #include <Time/TimeManager.hxx>
 #include <Environment/environment_mgr.hxx>
-#include <Environment/ephemeris.hxx>
 #include <GUI/gui.h>
 #include <GUI/new_gui.hxx>
 #include <MultiPlayer/multiplaymgr.hxx>
@@ -83,7 +82,7 @@
 #include "fg_os.hxx"
 #include "WindowSystemAdapter.hxx"
 #include <Main/viewer.hxx>
-
+#include <Main/fg_props.hxx>
 
 using namespace flightgear;
 
@@ -94,156 +93,10 @@ using std::cerr;
 // splash screen up and running right away.
 int idle_state = 0;
 
-
-void fgInitSoundManager();
-void fgSetNewSoundDevice(const char *);
-
 // The atexit() function handler should know when the graphical subsystem
 // is initialized.
 extern int _bootstrap_OSInit;
 
-// What should we do when we have nothing else to do?  Let's get ready
-// for the next move and update the display?
-static void fgMainLoop( void ) {
-    
-    static SGConstPropertyNode_ptr longitude
-        = fgGetNode("/position/longitude-deg");
-    static SGConstPropertyNode_ptr latitude
-        = fgGetNode("/position/latitude-deg");
-    static SGConstPropertyNode_ptr altitude
-        = fgGetNode("/position/altitude-ft");
-    static SGConstPropertyNode_ptr vn_fps
-        = fgGetNode("/velocities/speed-north-fps");
-    static SGConstPropertyNode_ptr ve_fps
-        = fgGetNode("/velocities/speed-east-fps");
-    static SGConstPropertyNode_ptr vd_fps
-        = fgGetNode("/velocities/speed-down-fps");
-      
-    static SGPropertyNode_ptr frame_signal
-        = fgGetNode("/sim/signals/frame", true);
-
-    static SGPropertyNode_ptr _statisticsFlag
-        = fgGetNode("/sim/timing-statistics/enabled", true);
-    static SGPropertyNode_ptr _statisticsInterval
-        = fgGetNode("/sim/timing-statistics/interval-s", true);
-    static SGPropertyNode_ptr _statiticsMinJitter
-        = fgGetNode("/sim/timing-statistics/min-jitter-ms", true);
-    static SGPropertyNode_ptr _statiticsMinTime
-        = fgGetNode("/sim/timing-statistics/min-time-ms", true);
-
-    frame_signal->fireValueChanged();
-    SGCloudLayer::enable_bump_mapping = fgGetBool("/sim/rendering/bump-mapping");
-    
-    SG_LOG( SG_ALL, SG_DEBUG, "Running Main Loop");
-    SG_LOG( SG_ALL, SG_DEBUG, "======= ==== ====");
-    
-    
-  // update "time"
-    double sim_dt, real_dt;
-    TimeManager* timeMgr = (TimeManager*) globals->get_subsystem("time");
-    // compute simulated time (allowing for pause, warp, etc) and
-    // real elapsed time
-    timeMgr->computeTimeDeltas(sim_dt, real_dt);
-
-    // update magvar model
-    globals->get_mag()->update( longitude->getDoubleValue()
-                                * SGD_DEGREES_TO_RADIANS,
-                                latitude->getDoubleValue()
-                                * SGD_DEGREES_TO_RADIANS,
-                                altitude->getDoubleValue() * SG_FEET_TO_METER,
-                                globals->get_time_params()->getJD() );
-
-    // Run ATC subsystem
-    globals->get_ATC_mgr()->update(sim_dt);
-    
-    globals->get_subsystem_mgr()->update(sim_dt);
-
-    // Update the sound manager last so it can use the CPU while the GPU
-    // is processing the scenery (doubled the frame-rate for me) -EMH-
-#ifdef ENABLE_AUDIO_SUPPORT
-    static bool smgr_init = true;
-    static SGPropertyNode *sound_working = fgGetNode("/sim/sound/working");
-    if (smgr_init == true) {
-        if (sound_working->getBoolValue() == true) {
-            fgInitSoundManager();
-            smgr_init = false;
-        }
-    } else {
-        static SGPropertyNode *sound_enabled = fgGetNode("/sim/sound/enabled");
-        static SGSoundMgr *smgr = globals->get_soundmgr();
-        static bool smgr_enabled = true;
-
-        if (sound_working->getBoolValue() == false) {	// request to reinit
-           smgr->reinit();
-           smgr->resume();
-           sound_working->setBoolValue(true);
-        }
-
-        if (smgr_enabled != sound_enabled->getBoolValue()) {
-            if (smgr_enabled == true) { // request to suspend
-                smgr->suspend();
-                smgr_enabled = false;
-            } else {
-                smgr->resume();
-                smgr_enabled = true;
-            }
-        }
-
-        if (smgr_enabled == true) {
-            static SGPropertyNode *volume = fgGetNode("/sim/sound/volume");
-            smgr->set_volume(volume->getFloatValue());
-            smgr->update(sim_dt);
-        }
-    }
-#endif
-
-    // END Tile Manager updates
-    bool scenery_loaded = fgGetBool("sim/sceneryloaded");
-    if (!scenery_loaded)
-    {
-        if (globals->get_tile_mgr()->isSceneryLoaded()
-             && fgGetBool("sim/fdm-initialized")) {
-            fgSetBool("sim/sceneryloaded",true);
-            fgSplashProgress("");
-            if (fgGetBool("/sim/sound/working")) {
-                globals->get_soundmgr()->activate();
-            }
-            globals->get_props()->tie("/sim/sound/devices/name",
-                  SGRawValueFunctions<const char *>(0, fgSetNewSoundDevice), false);
-        }
-        else
-        {
-            fgSplashProgress("loading scenery");
-            // be nice to loader threads while waiting for initial scenery, reduce to 2fps
-            simgear::sleepForMSec(500);
-        }
-    }
-
-    // print timing statistics
-    static bool _lastStatisticsFlag = false;
-    if (_lastStatisticsFlag != _statisticsFlag->getBoolValue())
-    {
-        // flag has changed, update subsystem manager
-        _lastStatisticsFlag = _statisticsFlag->getBoolValue();
-        globals->get_subsystem_mgr()->collectDebugTiming(_lastStatisticsFlag);
-    }
-    if (_lastStatisticsFlag)
-    {
-        static double elapsed = 0;
-        elapsed += real_dt;
-        if (elapsed >= _statisticsInterval->getDoubleValue())
-        {
-            // print and reset timing statistics
-            globals->get_subsystem_mgr()->printTimingStatistics(_statiticsMinTime->getDoubleValue(),
-                                                                _statiticsMinJitter->getDoubleValue());
-            elapsed = 0;
-        }
-    }
-
-    simgear::AtomicChangeListener::fireChangeListeners();
-
-    SG_LOG( SG_ALL, SG_DEBUG, "" );
-}
 
 void fgInitSoundManager()
 {
@@ -262,10 +115,116 @@ void fgInitSoundManager()
 
 void fgSetNewSoundDevice(const char *device)
 {
-    globals->get_soundmgr()->suspend();
-    globals->get_soundmgr()->stop();
-    globals->get_soundmgr()->init(device);
-    globals->get_soundmgr()->resume();
+    SGSoundMgr *smgr = globals->get_soundmgr();
+    smgr->suspend();
+    smgr->stop();
+    smgr->init(device);
+    smgr->resume();
+}
+
+// Update sound manager state (init/suspend/resume) and propagate property values,
+// since the sound manager doesn't read any properties itself.
+// Actual sound update is triggered by the subsystem manager.
+static void fgUpdateSound(double dt)
+{
+#ifdef ENABLE_AUDIO_SUPPORT
+    static bool smgr_init = true;
+    static SGPropertyNode *sound_working = fgGetNode("/sim/sound/working");
+    if (smgr_init == true) {
+        if (sound_working->getBoolValue() == true) {
+            fgInitSoundManager();
+            smgr_init = false;
+        }
+    } else {
+        static SGPropertyNode *sound_enabled = fgGetNode("/sim/sound/enabled");
+        static SGSoundMgr *smgr = globals->get_soundmgr();
+        static bool smgr_enabled = true;
+
+        if (sound_working->getBoolValue() == false) {   // request to reinit
+           smgr->reinit();
+           smgr->resume();
+           sound_working->setBoolValue(true);
+        }
+
+        if (smgr_enabled != sound_enabled->getBoolValue()) {
+            if (smgr_enabled == true) { // request to suspend
+                smgr->suspend();
+                smgr_enabled = false;
+            } else {
+                smgr->resume();
+                smgr_enabled = true;
+            }
+        }
+
+        if (smgr_enabled == true) {
+            static SGPropertyNode *volume = fgGetNode("/sim/sound/volume");
+            smgr->set_volume(volume->getFloatValue());
+        }
+    }
+#endif
+}
+
+static void fgLoadInitialScenery()
+{
+    static SGPropertyNode_ptr scenery_loaded
+        = fgGetNode("sim/sceneryloaded", true);
+
+    if (!scenery_loaded->getBoolValue())
+    {
+        if (globals->get_tile_mgr()->isSceneryLoaded()
+             && fgGetBool("sim/fdm-initialized")) {
+            fgSetBool("sim/sceneryloaded",true);
+            fgSplashProgress("");
+            if (fgGetBool("/sim/sound/working")) {
+                globals->get_soundmgr()->activate();
+            }
+            globals->get_props()->tie("/sim/sound/devices/name",
+                  SGRawValueFunctions<const char *>(0, fgSetNewSoundDevice), false);
+        }
+        else
+        {
+            fgSplashProgress("loading scenery");
+            // be nice to loader threads while waiting for initial scenery, reduce to 2fps
+            SGTimeStamp::sleepForMSec(500);
+        }
+    }
+}
+
+// What should we do when we have nothing else to do?  Let's get ready
+// for the next move and update the display?
+static void fgMainLoop( void )
+{
+    static SGPropertyNode_ptr frame_signal
+        = fgGetNode("/sim/signals/frame", true);
+
+    frame_signal->fireValueChanged();
+
+    SG_LOG( SG_GENERAL, SG_DEBUG, "Running Main Loop");
+    SG_LOG( SG_GENERAL, SG_DEBUG, "======= ==== ====");
+
+    // compute simulated time (allowing for pause, warp, etc) and
+    // real elapsed time
+    double sim_dt, real_dt;
+    TimeManager* timeMgr = (TimeManager*) globals->get_subsystem("time");
+    timeMgr->computeTimeDeltas(sim_dt, real_dt);
+
+    // update magvar model
+    globals->get_mag()->update( globals->get_aircraft_position(),
+                                globals->get_time_params()->getJD() );
+
+    // Propagate sound manager properties (note: actual update is triggered
+    // by the subsystem manager).
+    fgUpdateSound(sim_dt);
+
+    // update all subsystems
+    globals->get_subsystem_mgr()->update(sim_dt);
+
+    // END Tile Manager updates
+    fgLoadInitialScenery();
+
+    simgear::AtomicChangeListener::fireChangeListeners();
+
+    SG_LOG( SG_GENERAL, SG_DEBUG, "" );
 }
 
 // Operation for querying OpenGL parameters. This must be done in a
@@ -411,63 +370,27 @@ static void fgIdleFunction ( void ) {
 
     } else if ( idle_state == 6 ) {
         idle_state++;
-        // Initialize the sky
-
-        Ephemeris* eph = new Ephemeris;
-        globals->add_subsystem("ephemeris", eph);
-        eph->init(); // FIXME - remove this once SGSky code below is also a subsystem
-        eph->bind();
-
-        // TODO: move to environment mgr
-        thesky = new SGSky;
-        SGPath texture_path(globals->get_fg_root());
-        texture_path.append("Textures");
-        texture_path.append("Sky");
-        for (int i = 0; i < FGEnvironmentMgr::MAX_CLOUD_LAYERS; i++) {
-            SGCloudLayer * layer = new SGCloudLayer(texture_path.str());
-            thesky->add_cloud_layer(layer);
-        }
-
-        SGPath sky_tex_path( globals->get_fg_root() );
-        sky_tex_path.append( "Textures" );
-        sky_tex_path.append( "Sky" );
-        thesky->texture_path( sky_tex_path.str() );
-
-        // The sun and moon diameters are scaled down numbers of the
-        // actual diameters. This was needed to fit both the sun and the
-        // moon within the distance to the far clip plane.
-        // Moon diameter:    3,476 kilometers
-        // Sun diameter: 1,390,000 kilometers
-        thesky->build( 80000.0, 80000.0,
-                       463.3, 361.8,
-                       *globals->get_ephem(),
-                       fgGetNode("/environment", true));
-
+        
         // Initialize MagVar model
         SGMagVar *magvar = new SGMagVar();
         globals->set_mag( magvar );
-
-
-                                    // kludge to initialize mag compass
-                                    // (should only be done for in-flight
-                                    // startup)
+        
+        
+        // kludge to initialize mag compass
+        // (should only be done for in-flight
+        // startup)
         // update magvar model
         globals->get_mag()->update( fgGetDouble("/position/longitude-deg")
-                                    * SGD_DEGREES_TO_RADIANS,
-                                    fgGetDouble("/position/latitude-deg")
-                                    * SGD_DEGREES_TO_RADIANS,
-                                    fgGetDouble("/position/altitude-ft")
-                                    * SG_FEET_TO_METER,
-                                    globals->get_time_params()->getJD() );
+                                   * SGD_DEGREES_TO_RADIANS,
+                                   fgGetDouble("/position/latitude-deg")
+                                   * SGD_DEGREES_TO_RADIANS,
+                                   fgGetDouble("/position/altitude-ft")
+                                   * SG_FEET_TO_METER,
+                                   globals->get_time_params()->getJD() );
         double var = globals->get_mag()->get_magvar() * SGD_RADIANS_TO_DEGREES;
         fgSetDouble("/instrumentation/heading-indicator/offset-deg", -var);
         fgSetDouble("/instrumentation/heading-indicator-fg/offset-deg", -var);
 
-
-        // airport = new ssgBranch;
-        // airport->setName( "Airport Lighting" );
-        // lighting->addKid( airport );
-
         fgSplashProgress("initializing subsystems");
 
     } else if ( idle_state == 7 ) {
@@ -491,7 +414,11 @@ static void fgIdleFunction ( void ) {
             string command = "mpg123 " + mp3file.str() + "> /dev/null 2>&1";
 # endif
 
-            system ( command.c_str() );
+            if (0 != system ( command.c_str() ))
+            {
+                SG_LOG( SG_SOUND, SG_WARN,
+                    "Failed to play mp3 file " << mp3file.str() << ". Maybe mp3 player is not installed." );
+            }
         }
 #endif
         // This is the top level init routine which calls all the
@@ -510,6 +437,8 @@ static void fgIdleFunction ( void ) {
         // runway selection as for AI traffic. However, this code belongs to
         // somewhere(?) else - if I only new where...
         if( true == fgGetBool( "/environment/metar/valid" ) ) {
+            SG_LOG(SG_ENVIRONMENT, SG_INFO,
+                "Using METAR for runway selection: '" << fgGetString("/environment/metar/data") << "'" );
             // the realwx_ctrl fetches metar in the foreground on init,
             // If it was able to fetch a metar or one was given on the commandline,
             // the valid flag is set here, otherwise it is false
@@ -525,6 +454,9 @@ static void fgIdleFunction ( void ) {
                 extern bool fgSetPosFromAirportIDandHdg( const string& id, double tgt_hdg );
                 fgSetPosFromAirportIDandHdg( apt, hdg );
             }
+        } else {
+            SG_LOG(SG_ENVIRONMENT, SG_INFO,
+                "No METAR available to pick active runway" );
         }
 
         fgSplashProgress("initializing graphics engine");
@@ -551,11 +483,6 @@ static void fgIdleFunction ( void ) {
     }
 }
 
-static void fgWinResizeFunction(int width, int height)
-{
-    globals->get_renderer()->resize(width, height);
-}
-
 static void upper_case_property(const char *name)
 {
     using namespace simgear;
@@ -588,7 +515,7 @@ int fgMainInit( int argc, char **argv ) {
 #endif
     SG_LOG( SG_GENERAL, SG_INFO, "FlightGear:  Version "
             << version );
-    SG_LOG( SG_GENERAL, SG_INFO, "Built with " << SG_COMPILER_STR << endl );
+  SG_LOG( SG_GENERAL, SG_INFO, "Built with " << SG_COMPILER_STR << std::endl );
 
     // Allocate global data structures.  This needs to happen before
     // we parse command line options
@@ -610,44 +537,19 @@ int fgMainInit( int argc, char **argv ) {
     upper_case_property("/sim/tower/airport-id");
     upper_case_property("/autopilot/route-manager/input");
 
-    // Scan the config file(s) and command line options to see if
-    // fg_root was specified (ignore all other options for now)
-    fgInitFGRoot(argc, argv);
-
-    // Check for the correct base package version
-    static char required_version[] = "2.4.0";
-    string base_version = fgBasePackageVersion();
-    if ( !(base_version == required_version) ) {
-        // tell the operator how to use this application
-
-        SG_LOG( SG_GENERAL, SG_ALERT, "" ); // To popup the console on windows
-        cerr << endl << "Base package check failed:" << endl \
-             << "  Version " << base_version << " found at: " \
-             << globals->get_fg_root() << endl \
-             << "  Version " << required_version << " is required." << endl \
-             << "Please upgrade/downgrade base package." << endl;
-#ifdef _MSC_VER
-        cerr << "Hit a key to continue..." << endl;
-        cin.get();
-#endif
-        exit(-1);
-    }
-
     // Load the configuration parameters.  (Command line options
     // override config file options.  Config file options override
     // defaults.)
     if ( !fgInitConfig(argc, argv) ) {
-        SG_LOG( SG_GENERAL, SG_ALERT, "Config option parsing failed ..." );
-        exit(-1);
+      SG_LOG( SG_GENERAL, SG_ALERT, "Config option parsing failed ..." );
+      exit(-1);
     }
 
     // Initialize the Window/Graphics environment.
     fgOSInit(&argc, argv);
     _bootstrap_OSInit++;
 
-    fgRegisterWindowResizeHandler( &fgWinResizeFunction );
     fgRegisterIdleHandler( &fgIdleFunction );
-    fgRegisterDrawHandler( &FGRenderer::update );
 
     // Initialize sockets (WinSock needs this)
     simgear::Socket::initSockets();
