@@ -31,9 +31,10 @@
 #include <Main/fg_props.hxx>
 #include <Main/util.hxx>
 #include <Scenery/scenery.hxx>
-#include <Navaids/navrecord.hxx>
+#include <Navaids/navlist.hxx>
 #include <Navaids/procedure.hxx>
 
+
 #include "NasalSys.hxx"
 
 static FGNasalSys* nasalSys = 0;
@@ -214,7 +215,7 @@ static naRef f_getprop(naContext c, naRef me, int argc, naRef* args)
         {
         double dv = p->getDoubleValue();
         if (osg::isNaN(dv)) {
-          SG_LOG(SG_GENERAL, SG_ALERT, "Nasal getprop: property " << p->getPath() << " is NaN");
+          SG_LOG(SG_NASAL, SG_ALERT, "Nasal getprop: property " << p->getPath() << " is NaN");
           return naNil();
         }
         
@@ -293,7 +294,7 @@ static naRef f_print(naContext c, naRef me, int argc, naRef* args)
         if(naIsNil(s)) continue;
         buf += naStr_data(s);
     }
-    SG_LOG(SG_GENERAL, SG_ALERT, buf);
+    SG_LOG(SG_NASAL, SG_ALERT, buf);
     return naNum(buf.length());
 }
 
@@ -695,6 +696,92 @@ static naRef f_airportinfo(naContext c, naRef me, int argc, naRef* args)
 }
 
 
+// Returns vector of data hash for navaid of a <type>, nil on error
+// navaids sorted by ascending distance 
+// navinfo([<lat>,<lon>],[<type>],[<id>])
+// lat/lon (numeric): use latitude/longitude instead of ac position
+// type:              ("fix"|"vor"|"ndb"|"ils"|"dme"|"tacan"|"any")
+// id:                (partial) id of the fix
+// examples:
+// navinfo("vor")     returns all vors
+// navinfo("HAM")     return all navaids who's name start with "HAM"
+// navinfo("vor", "HAM") return all vor who's name start with "HAM"
+//navinfo(34,48,"vor","HAM") return all vor who's name start with "HAM" 
+//                           sorted by distance relative to lat=34, lon=48
+static naRef f_navinfo(naContext c, naRef me, int argc, naRef* args)
+{
+    static SGConstPropertyNode_ptr latn = fgGetNode("/position/latitude-deg", true);
+    static SGConstPropertyNode_ptr lonn = fgGetNode("/position/longitude-deg", true);
+    SGGeod pos;
+
+    if(argc >= 2 && naIsNum(args[0]) && naIsNum(args[1])) {
+        pos = SGGeod::fromDeg(args[1].num, args[0].num);
+        args += 2;
+        argc -= 2;
+    } else {
+        pos = SGGeod::fromDeg(lonn->getDoubleValue(), latn->getDoubleValue());
+    }
+
+    FGPositioned::Type type = FGPositioned::INVALID;
+    nav_list_type navlist;
+    const char * id = "";
+
+    if(argc > 0 && naIsString(args[0])) {
+        const char *s = naStr_data(args[0]);
+        if(!strcmp(s, "any")) type = FGPositioned::INVALID;
+        else if(!strcmp(s, "fix")) type = FGPositioned::FIX;
+        else if(!strcmp(s, "vor")) type = FGPositioned::VOR;
+        else if(!strcmp(s, "ndb")) type = FGPositioned::NDB;
+        else if(!strcmp(s, "ils")) type = FGPositioned::ILS;
+        else if(!strcmp(s, "dme")) type = FGPositioned::DME;
+        else if(!strcmp(s, "tacan")) type = FGPositioned::TACAN;
+        else id = s; // this is an id
+        ++args;
+        --argc;
+    } 
+
+    if(argc > 0 && naIsString(args[0])) {
+        if( *id != 0 ) {
+            naRuntimeError(c, "navinfo() called with navaid id");
+            return naNil();
+        }
+        id = naStr_data(args[0]);
+        ++args;
+        --argc;
+    }
+
+    if( argc > 0 ) {
+        naRuntimeError(c, "navinfo() called with too many arguments");
+        return naNil();
+    }
+
+    navlist = globals->get_navlist()->findByIdentAndFreq( pos, id, 0.0, type );
+
+    naRef reply = naNewVector(c);
+    for( nav_list_type::const_iterator it = navlist.begin(); it != navlist.end(); ++it ) {
+        const FGNavRecord * nav = *it;
+
+        // set navdata hash
+        naRef navdata = naNewHash(c);
+#define HASHSET(s,l,n) naHash_set(navdata, naStr_fromdata(naNewString(c),s,l),n)
+        HASHSET("id", 2, naStr_fromdata(naNewString(c),
+            const_cast<char *>(nav->ident().c_str()), nav->ident().length()));
+        HASHSET("name", 4, naStr_fromdata(naNewString(c),
+            const_cast<char *>(nav->name().c_str()), nav->name().length()));
+        HASHSET("frequency", 9, naNum(nav->get_freq()));
+        HASHSET("lat", 3, naNum(nav->get_lat()));
+        HASHSET("lon", 3, naNum(nav->get_lon()));
+        HASHSET("elevation", 9, naNum(nav->get_elev_ft() * SG_FEET_TO_METER));
+        HASHSET("type", 4, naStr_fromdata(naNewString(c),
+            const_cast<char *>(nav->nameForType(nav->type())), strlen(nav->nameForType(nav->type()))));
+        HASHSET("distance", 8, naNum(SGGeodesy::distanceNm( pos, nav->geod() ) * SG_NM_TO_METER ) );
+        HASHSET("bearing", 7, naNum(SGGeodesy::courseDeg( pos, nav->geod() ) ) );
+#undef HASHSET
+        naVec_append( reply, navdata );
+    }
+    return reply;
+}
+
 // Table of extension functions.  Terminate with zeros.
 static struct { const char* name; naCFunction func; } funcs[] = {
     { "getprop",   f_getprop },
@@ -717,6 +804,7 @@ static struct { const char* name; naCFunction func; } funcs[] = {
     { "geodtocart", f_geodtocart },
     { "geodinfo", f_geodinfo },
     { "airportinfo", f_airportinfo },
+    { "navinfo", f_navinfo },
     { 0, 0 }
 };
 
@@ -790,6 +878,19 @@ void FGNasalSys::update(double)
         _dead_listener.clear();
     }
 
+    if (!_loadList.empty())
+    {
+        // process Nasal load hook (only one per update loop to avoid excessive lags)
+        _loadList.pop()->load();
+    }
+    else
+    if (!_unloadList.empty())
+    {
+        // process pending Nasal unload hooks after _all_ load hooks were processed
+        // (only unload one per update loop to avoid excessive lags)
+        _unloadList.pop()->unload();
+    }
+
     // The global context is a legacy thing.  We use dynamically
     // created contexts for naCall() now, so that we can call them
     // recursively.  But there are still spots that want to use it for
@@ -1139,7 +1240,7 @@ naRef FGNasalSys::setListener(naContext c, int argc, naRef* args)
     FGNasalListener *nl = new FGNasalListener(node, code, this,
             gcSave(code), _listenerId, init, type);
 
-    node->addChangeListener(nl, init);
+    node->addChangeListener(nl, init != 0);
 
     _listener[_listenerId] = nl;
     return naNum(_listenerId++);
@@ -1270,36 +1371,26 @@ bool FGNasalListener::changed(SGPropertyNode* node)
 
 unsigned int FGNasalModelData::_module_id = 0;
 
-void FGNasalModelData::modelLoaded(const string& path, SGPropertyNode *prop,
-                                   osg::Node *)
+void FGNasalModelData::load()
 {
-    if(!prop)
-        return;
-    SGPropertyNode *nasal = prop->getNode("nasal");
-    if(!nasal)
-        return;
-
-    SGPropertyNode *load = nasal->getNode("load");
-    _unload = nasal->getNode("unload");
-    if(!load && !_unload)
-        return;
-
     std::stringstream m;
     m << "__model" << _module_id++;
     _module = m.str();
 
-    const char *s = load ? load->getStringValue() : "";
+    SG_LOG(SG_NASAL, SG_DEBUG, "Loading nasal module " << _module.c_str());
+
+    const char *s = _load ? _load->getStringValue() : "";
 
     naRef arg[2];
     arg[0] = nasalSys->propNodeGhost(_root);
-    arg[1] = nasalSys->propNodeGhost(prop);
-    nasalSys->createModule(_module.c_str(), path.c_str(), s, strlen(s),
+    arg[1] = nasalSys->propNodeGhost(_prop);
+    nasalSys->createModule(_module.c_str(), _path.c_str(), s, strlen(s),
                            _root, 2, arg);
 }
 
-FGNasalModelData::~FGNasalModelData()
+void FGNasalModelData::unload()
 {
-    if(_module.empty())
+    if (_module.empty())
         return;
 
     if(!nasalSys) {
@@ -1308,14 +1399,52 @@ FGNasalModelData::~FGNasalModelData()
         return;
     }
 
-    if(_unload) {
+    SG_LOG(SG_NASAL, SG_DEBUG, "Unloading nasal module " << _module.c_str());
+
+    if (_unload)
+    {
         const char *s = _unload->getStringValue();
         nasalSys->createModule(_module.c_str(), _module.c_str(), s, strlen(s), _root);
     }
+
     nasalSys->deleteModule(_module.c_str());
 }
 
+void FGNasalModelDataProxy::modelLoaded(const string& path, SGPropertyNode *prop,
+                                   osg::Node *)
+{
+    if(!nasalSys) {
+        SG_LOG(SG_NASAL, SG_WARN, "Trying to run a <load> script "
+                "without Nasal subsystem present.");
+        return;
+    }
 
+    if(!prop)
+        return;
+
+    SGPropertyNode *nasal = prop->getNode("nasal");
+    if(!nasal)
+        return;
+
+    SGPropertyNode* load   = nasal->getNode("load");
+    SGPropertyNode* unload = nasal->getNode("unload");
+
+    if ((!load) && (!unload))
+        return;
+
+    _data = new FGNasalModelData(_root, path, prop, load, unload);
+
+    // register Nasal module to be created and loaded in the main thread.
+    nasalSys->registerToLoad(_data);
+}
+
+FGNasalModelDataProxy::~FGNasalModelDataProxy()
+{
+    // when necessary, register Nasal module to be destroyed/unloaded
+    // in the main thread.
+    if ((_data.valid())&&(nasalSys))
+        nasalSys->registerToUnload(_data);
+}
 
 // NasalXMLVisitor class: handles EasyXML visitor callback for parsexml()
 //
